//$Id: ManagedHibernateSession.java,v 1.15 2006/05/26 01:49:21 gavin Exp $
package org.jboss.seam.core;

import static org.jboss.seam.InterceptionType.NEVER;

import java.io.Serializable;

import javax.naming.NamingException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.jboss.seam.Component;
import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Create;
import org.jboss.seam.annotations.Destroy;
import org.jboss.seam.annotations.Intercept;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.Unwrap;
import org.jboss.seam.util.Naming;

/**
 * A Seam component that manages a conversation-scoped extended
 * persistence context that can be shared by arbitrary other
 * components.
 * 
 * @author Gavin King
 */
@Scope(ScopeType.CONVERSATION)
@Intercept(NEVER)
public class ManagedHibernateSession implements Serializable
{
   //TODO: a conversation-scope mutable component, could break in a cluster
   
   /** The serialVersionUID */
   private static final long serialVersionUID = 3130309555079841107L;

   private static final Log log = LogFactory.getLog(ManagedHibernateSession.class);
   
   private Session session;
   private String sessionFactoryJndiName;
   private String componentName;
   
   @Create
   public void create(Component component)
   {
      this.componentName = component.getName();
      if (sessionFactoryJndiName==null)
      {
         sessionFactoryJndiName = "java:/" + componentName;
      }
      try
      {
         session = getSessionFactory().openSession();
      }
      catch (NamingException ne)
      {
         throw new IllegalArgumentException("SessionFactory not found", ne);
      }
      
      log.debug("created seam managed session for session factory: "+ sessionFactoryJndiName);
   }
   
   @Unwrap
   public Session getSession()
   {
      session.isOpen();
      return session;
   }
   
   @Destroy
   public void destroy()
   {
      log.debug("destroying seam managed session for session factory: " + sessionFactoryJndiName);
      session.close();
   }
   
   private SessionFactory getSessionFactory()
         throws NamingException
   {
      return (SessionFactory) Naming.getInitialContext().lookup(sessionFactoryJndiName);
   }
   
   public String toString()
   {
      return "ManagedHibernateSession(" + sessionFactoryJndiName + ")";
   }

   public String getSessionFactoryJndiName()
   {
      return sessionFactoryJndiName;
   }

   public void setSessionFactoryJndiName(String sessionFactoryName)
   {
      this.sessionFactoryJndiName = sessionFactoryName;
   }

   public String getComponentName() {
      return componentName;
   }
   
}
